/*  FeedListAssistant - NEWS
        
    Copyright 2009 Palm, Inc.  All rights reserved.
    
    Main scene for News app. Includes AddDialog-assistant for handling
    feed entry and then feedlist-assistant and supporting functions.
    
    Major components:
    - AddDialogAssisant; Scene assistant for add feed dialog and handlers
    - FeedListAssistant; manages feedlists
    - List Handlers - delete, reorder and add feeds
        
    Arguments:
    - feeds; Feeds object

*/

//    ---------------------------------------------------------------------------------------
//
//    FeedListAssistant - main scene handler for news feedlists
//    
function FeedListAssistant(feeds) {
    this.feeds = feeds;
}

FeedListAssistant.prototype.setup =  function() {
    
    // Setup the feed list, but it's empty
    this.controller.setupWidget("feedListWgt",
         {
            itemTemplate:"feedList/feedRowTemplate", 
            listTemplate:"feedList/feedListTemplate", 
            addItemLabel:"Add...", 
            swipeToDelete:true, 
            renderLimit: 40,
            reorderable:true
        },
        this.feedWgtModel = {items: this.feeds.list});
    
    // Setup event handlers: list selection, add, delete and reorder feed entry
    this.showFeedHandler = this.showFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listTap,
        this.showFeedHandler);
    this.addNewFeedHandler = this.addNewFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listAdd,
        this.addNewFeedHandler);    
    this.listDeleteFeedHandler = this.listDeleteFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listDelete,
        this.listDeleteFeedHandler);
    this.listReorderFeedHandler = this.listReorderFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listReorder,
        this.listReorderFeedHandler); 
    
    //  Setup Drawer for add Feed; closed to start
	this.controller.setupWidget('feedDrawer', {}, this.addDrawerModel={open :false});

    // Set the add feed drawer title to either Edit or Add Feed
    var addFeedTitleElement = this.controller.get("add-feed-title");
    addFeedTitleElement.innerHTML = "Add News Feed Source";
        
    // Setup text field for the add new feed's URL
    this.controller.setupWidget(
        "newFeedURL",
        {
              hintText: "RSS or ATOM feed URL",
              autoFocus: true,
              limitResize: true,
              autoReplace: false,
              textCase: Mojo.Widget.steModeLowerCase,
              enterSubmits: false
        },
        this.urlModel = {value : ""});
    
    // Setup text field for the new feed's name    
    this.controller.setupWidget(
        "newFeedName",
        {
              hintText: "Title (Optional)",
              limitResize: true,
              autoReplace: false,
              textCase: Mojo.Widget.steModeTitleCase,
              enterSubmits: false
        },
        this.nameModel = {value : ""});
    
    // Setup OK  & Cancel buttons
    //   OK button is an activity button which will be active
    //   while processing and adding feed. Cancel is an HTML button
    //   which will just cancel the action and close the scene
    this.okButtonModel = {label: $L("OK"), disabled: false};
    this.controller.setupWidget("okButton", {type: Mojo.Widget.activityButton},
        this.okButtonModel);
    this.okButtonActive = false;
    this.okButton = this.controller.get("okButton");
    this.checkFeedHandler = this.checkFeed.bindAsEventListener(this);
    this.controller.listen("okButton", Mojo.Event.tap,
      this.checkFeedHandler);
      
    this.cancelButtonModel = {label: $L("Cancel"), disabled: false};
    this.controller.setupWidget("cancelButton", {type: Mojo.Widget.defaultButton},
        this.cancelButtonModel);
    this.closeAddFeedHandler = this.closeAddFeed.bindAsEventListener(this);
    this.controller.listen("cancelButton", Mojo.Event.tap,
      this.closeAddFeedHandler);
    
    // Update the feed list
    this.feeds.updateFeedList();
};

// activate
FeedListAssistant.prototype.activate =  function() {
    this.feeds.registerListModel(this, this.feedWgtModel);
    
    if (News.feedListChanged === true)    {
        this.feedWgtModel.items = this.feeds.list; 
        this.controller.modelChanged(this.feedWgtModel, this);
        this.controller.modelChanged(this.searchFieldModel, this);
    } 
};

// deactivate
FeedListAssistant.prototype.deactivate =  function() {
    Mojo.Log.info("FeedList deactivating");
    this.feeds.removeListModel(this.feedWgtModel);
};

// cleanup - always turn off timers, and save this.feeds.list contents
FeedListAssistant.prototype.cleanup =  function() {
    Mojo.Log.info("FeedList cleaning up");
    
    // Remove event listeners
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listTap, this.showFeedHandler);
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listAdd, this.addNewFeedHandler);    
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listDelete, this.listDeleteFeedHandler);
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listReorder, this.listReorderFeedHandler);
    this.controller.stopListening("okButton", Mojo.Event.tap,
        this.checkFeedHandler);
    this.controller.stopListening("cancelButton", Mojo.Event.tap,
        this.cancelHandler);
};

// ------------------------------------------------------------------------
// List functions for Delete, Reorder and Add
//
// listDeleteFeed - triggered by deleting a feed from the list and updates
// the feedlist to reflect the deletion
//
FeedListAssistant.prototype.listDeleteFeed =  function(event) {
    Mojo.log("News deleting "+event.item.title+".");
    
    var deleteIndex = this.feeds.list.indexOf(event.item);
    this.feeds.list.splice(deleteIndex, 1);
    this.feedWgtModel.items = this.feeds.list;
    News.feedListChanged = true;
};

// listReorderFeed - triggered re-ordering feed list and updates the
// feedlist to reflect the changed order
FeedListAssistant.prototype.listReorderFeed =  function(event) {
    Mojo.log("com.palm.app.news - News moving "+event.item.title+".");
    
    var fromIndex = this.feeds.list.indexOf(event.item);
    var toIndex = event.toIndex;
    this.feeds.list.splice(fromIndex, 1);
    this.feeds.list.splice(toIndex, 0, event.item);    
    this.feedWgtModel.items = this.feeds.list;
    News.feedListChanged = true;
};

// addNewFeed - triggered by "Add..." item in feed list
FeedListAssistant.prototype.addNewFeed = function() {
    this.addDrawerModel.open = true;
    this.controller.modelChanged(this.addDrawerModel); 
};

// ------------------------------------------------------------------------
// Add Feed Functions - set of functions to add new feeds to the list
//   when the "Add..." is selected on the feedlist. The drawer will
//   allow the user to enter the feed's url and optionally a name. When
//   the "Ok" button is tapped, the new feed will be loaded. If no errors
//   are encountered, the dialog will close otherwise the error will be
//   posted and the user encouraged to try again.
// 


// checkFeed  - called when OK button is clicked
FeedListAssistant.prototype.checkFeed = function() {
        
        if (this.okButtonActive === true)  {
            // Shouldn't happen, but log event if it does and exit
            Mojo.Log.info("Multiple Check Feed requests");
            return;
        }
        
        // Check entered URL and name to confirm that it is a valid feedlist
        Mojo.Log.info("New Feed URL Request: ", this.urlModel.value);
        
        // Check for "http://" on front or other legal prefix; any string of
        // 1 to 5 alpha characters followed by ":" is ok, else prepend "http://"        
        var    url = this.urlModel.value; 
        if (/^[a-z]{1,5}:/.test(url) === false)    {
            // Strip any leading slashes
            url = url.replace(/^\/{1,2}/,"");                                
            url = "http://"+url;                                                        
        }
        
        // Update the entered URL & model
        this.urlModel.value = url;
        this.controller.modelChanged(this.urlModel);
            
        this.okButton.mojo.activate();
        this.okButtonActive = true;
        this.okButtonModel.label = "Updating Feed";
        this.okButtonModel.disabled = true;
        this.controller.modelChanged(this.okButtonModel);

        var request = new Ajax.Request(url, {
            method: "get",
            evalJSON: "false",
            onSuccess: this.checkSuccess.bind(this),
            onFailure: this.checkFailure.bind(this)
        });

};

// checkSuccess - Ajax request failure
FeedListAssistant.prototype.checkSuccess = function(transport) {
        Mojo.Log.info("Valid URL - HTTP Status", transport.status);

        
        // DEBUG - Work around due occasion Ajax XML error in response.
        if (transport.responseXML === null && transport.responseText !== null) {
                Mojo.Log.info("Request not in XML format - manually converting");
                transport.responseXML = new DOMParser().parseFromString(transport.responseText, "text/xml");
         }
        
        var feedError = News.errorNone;
        
        this.feeds.list.push({title:this.nameModel.value, url:this.urlModel.value,
            type:"", value:false, numUnRead:0, stories:[]});
        feedError = this.feeds.processFeed(transport);                

        // If successful processFeed returns errorNone
        if (feedError === News.errorNone)    {
            // update the list widget and exit
            this.feedWgtModel.items = this.feeds.list;       
            this.controller.modelChanged(this.feedWgtModel);
            this.closeAddFeed();
            
        }
        else    {
            // Feed can't be processed - remove it but keep the dialog open
            this.feeds.list.pop();                                                                        
            if (feedError == News.invalidFeedError)    {
                Mojo.Log.warn("Feed ",
                    this.urlModel.value, " isn't a supported feed type.");
                var addFeedTitleElement = this.controller.get("add-feed-title");
                addFeedTitleElement.innerHTML = "Invalid Feed Type - Please Retry";
            }
        }
        
        // clear OK button state
        this.okButton.mojo.deactivate();
        this.okButtonActive = false;
        this.okButtonModel.label = "OK";
        this.okButtonModel.disabled = false;
        this.controller.modelChanged(this.okButtonModel);        
};

// checkFailure  - Ajax request failure
FeedListAssistant.prototype.checkFailure = function(transport) {
        // Log error and put message in status area
        Mojo.Log.info("Invalid URL - HTTP Status", transport.status);
        var addFeedTitleElement = this.controller.get("add-feed-title");
        addFeedTitleElement.innerHTML = "Invalid URL - Please Retry.";
};

// closeAddFeed  - close Drawer
FeedListAssistant.prototype.closeAddFeed = function() {
        // Close Drawer and clear text fields
        this.addDrawerModel.open = false;
        this.controller.modelChanged(this.addDrawerModel);
        
        //  Clear text fields for next time
        this.nameModel.value = "";
        this.controller.modelChanged(this.nameModel);   
        this.urlModel.value = "";
        this.controller.modelChanged(this.urlModel);   
};


// ---------------------------------------------------------------------------------
// Show feed handler
//
// showFeed - triggered by tapping a feed in the this.feeds.list.
FeedListAssistant.prototype.showFeed = function(event) {
    Mojo.Controller.stageController.pushScene("storyList", this.feeds.list, event.index);
};

