/*  FeedListAssistant - NEWS
        
    Copyright 2009 Palm, Inc.  All rights reserved.
    
    Main scene for News app. Includes AddDialog-assistant for handling
    feed entry and then feedlist-assistant and supporting functions.
    
    Major components:
    - AddDialogAssisant; Scene assistant for add feed dialog and handlers
    - FeedListAssistant; manages feedlists
    - List Handlers - delete, reorder and add feeds
        
    Arguments:
    - feeds; Feeds object

*/


// ------------------------------------------------------------------------
// AddDialogAssistant - simple controller for adding new feeds to the list
//   when the "Add..." is selected on the feedlist. The dialog will
//   allow the user to enter the feed's url and optionally a name. When
//   the "Ok" button is tapped, the new feed will be loaded. If no errors
//   are encountered, the dialog will close otherwise the error will be
//   posted and the user encouraged to try again.
// 
function AddDialogAssistant(sceneAssistant, feeds, index) {
    this.feeds = feeds;
    this.sceneAssistant = sceneAssistant;
        
    //  If an index is provided then this is an edit feed, not add feed
    //  so provide the existing title, url and modify the dialog title
    if (index !== undefined) {
        this.title = this.feeds.list[index].title;
        this.url = this.feeds.list[index].url;
        this.feedIndex = index;
        this.dialogTitle = "Edit News Feed";
    }
    else {
        this.title = "";
        this.url = "";
        this.feedIndex = null;
        this.dialogTitle = "Add News Feed Source";
    }
}
    
AddDialogAssistant.prototype.setup = function(widget) {
        this.widget = widget;
        
        // Set the dialog title to either Edit or Add Feed
        var addFeedTitleElement = this.sceneAssistant.controller.get("add-feed-title");
        addFeedTitleElement.innerHTML = this.dialogTitle;
            
        // Setup text field for the new feed's URL
        this.sceneAssistant.controller.setupWidget(
            "newFeedURL",
            {
                  hintText: "RSS or ATOM feed URL",
                  autoFocus: true,
                  limitResize: true,
                  autoReplace: false,
                  textCase: Mojo.Widget.steModeLowerCase,
                  enterSubmits: false
            },
            this.urlModel = {value : this.url});
        
        // Setup text field for the new feed's name    
        this.sceneAssistant.controller.setupWidget(
            "newFeedName",
            {
                    hintText: "Title (Optional)",
                      limitResize: true,
                      autoReplace: false,
                      textCase: Mojo.Widget.steModeTitleCase,
                      enterSubmits: false
            },
            this.nameModel = {value : this.title});
        
        // Setup OK  & Cancel buttons
        //   OK button is an activity button which will be active
        //   while processing and adding feed. Cancel will just
        //   close the scene
        this.okButtonModel = {label: "OK", disabled: false};
        this.sceneAssistant.controller.setupWidget("okButton", {type: Mojo.Widget.activityButton},
            this.okButtonModel);
        this.okButtonActive = false;
        this.okButton = this.sceneAssistant.controller.get("okButton");
        this.checkFeedHandler = this.checkFeed.bindAsEventListener(this);
        this.sceneAssistant.controller.listen("okButton", Mojo.Event.tap,
          this.checkFeedHandler);

        this.cancelButtonModel = {label: "Cancel", disabled: false};
        this.sceneAssistant.controller.setupWidget("cancelButton", {type: Mojo.Widget.defaultButton},
          this.cancelButtonModel);
        this.sceneAssistant.controller.listen("cancelButton", Mojo.Event.tap,
          this.widget.mojo.close);
};

// checkFeed  - called when OK button is clicked
AddDialogAssistant.prototype.checkFeed = function() {
        
        if (this.okButtonActive === true)  {
            // Shouldn't happen, but log event if it does and exit
            Mojo.Log.info("Multiple Check Feed requests");
            return;
        }
        
        // Check entered URL and name to confirm that it is a valid feedlist
        Mojo.Log.info("New Feed URL Request: ", this.urlModel.value);
        
        // Check for "http://" on front or other legal prefix; any string of
        // 1 to 5 alpha characters followed by ":" is ok, else prepend "http://"        
        var    url = this.urlModel.value; 
        if (/^[a-z]{1,5}:/.test(url) === false)    {
            // Strip any leading slashes
            url = url.replace(/^\/{1,2}/,"");                                
            url = "http://"+url;                                                        
        }
        
        // Update the entered URL & model
        this.urlModel.value = url;
        this.sceneAssistant.controller.modelChanged(this.urlModel);
        
        // If the url is the same, then assume that it's just a title change,
        // update the feed title and close the dialog. Otherwise update the feed.
        if (this.feedIndex && this.feeds.list[this.feedIndex].url == this.urlModel.value) {
            this.feeds.list[this.feedIndex].title = this.nameModel.value;
            this.sceneAssistant.feedWgtModel.items = this.feeds.list;
            this.sceneAssistant.controller.modelChanged(this.sceneAssistant.feedWgtModel);
            this.widget.mojo.close();
        }
        else {
            
            this.okButton.mojo.activate();
            this.okButtonActive = true;
            this.okButtonModel.label = "Updating Feed";
            this.okButtonModel.disabled = true;
            this.sceneAssistant.controller.modelChanged(this.okButtonModel);

            var request = new Ajax.Request(url, {
                method: "get",
                evalJSON: "false",
                onSuccess: this.checkSuccess.bind(this),
                onFailure: this.checkFailure.bind(this)
            });
        }
};

// checkSuccess - Ajax request failure
AddDialogAssistant.prototype.checkSuccess = function(transport) {
        Mojo.Log.info("Valid URL - HTTP Status", transport.status);

        
        // DEBUG - Work around due occasion Ajax XML error in response.
        if (transport.responseXML === null && transport.responseText !== null) {
                Mojo.Log.info("Request not in XML format - manually converting");
                transport.responseXML = new DOMParser().parseFromString(transport.responseText, "text/xml");
         }
        
        var feedError = News.errorNone;
        
        //  If a new feed, push the entered feed data on to the feedlist and
        //  call processFeed to evaluate it.
        if (this.feedIndex === null) {
            this.feeds.list.push({title:this.nameModel.value, url:this.urlModel.value,
                type:"", value:false, numUnRead:0, stories:[]});
            // processFeed - index defaults to last entry
            feedError = this.feeds.processFeed(transport);                
        }
        else    {
            this.feeds.list[this.feedIndex] = {title:this.nameModel.value, url:this.urlModel.value,
                type:"", value:false, numUnRead:0, stories:[]};
            feedError = this.feeds.processFeed(transport, this.feedIndex);    
        }
        
        // If successful processFeed returns errorNone
        if (feedError === News.errorNone)    {
            // update the widget, save the DB and exit
            this.sceneAssistant.feedWgtModel.items = this.feeds.list;       
            this.sceneAssistant.controller.modelChanged(this.sceneAssistant.feedWgtModel);    
            this.widget.mojo.close();    
        }
        else    {
            // Feed can't be processed - remove it but keep the dialog open
            this.feeds.list.pop();                                                                        
            if (feedError == News.invalidFeedError)    {
                Mojo.Log.warn("Feed ",
                    this.urlModel.value, " isn't a supported feed type.");
                var addFeedTitleElement = this.sceneAssistant.controller.get("add-feed-title");
                addFeedTitleElement.innerHTML = "Invalid Feed Type - Please Retry";
            }
            
            this.okButton.mojo.deactivate();
            this.okButtonActive = false;
            this.okButtonModel.label = "OK";
            this.okButtonModel.disabled = false;
            this.sceneAssistant.controller.modelChanged(this.okButtonModel);        
        }
};

// checkFailure  - Ajax request failure
AddDialogAssistant.prototype.checkFailure = function(transport) {
        // Log error and put message in status area
        Mojo.Log.info("Invalid URL - HTTP Status", transport.status);
        var addFeedTitleElement = this.sceneAssistant.controller.get("add-feed-title");
        addFeedTitleElement.innerHTML = "Invalid URL - Please Retry.";
};

// cleanup  - close Dialog
AddDialogAssistant.prototype.cleanup = function() {
        // TODO - Cancel Ajax request or Feed operation if in progress
        this.sceneAssistant.controller.stopListening("okButton", Mojo.Event.tap,
            this.checkFeedHandler);
        this.sceneAssistant.controller.stopListening("cancelButton", Mojo.Event.tap,
            this.widget.mojo.close);
};


//    ---------------------------------------------------------------------------------------
//
//    FeedListAssistant - main scene handler for news feedlists
//    
function FeedListAssistant(feeds) {
    this.feeds = feeds;
}

FeedListAssistant.prototype.setup =  function() {

    // Setup App Menu
    this.controller.setupWidget(Mojo.Menu.appMenu, News.MenuAttr, News.MenuModel);
    
    // Setup the feed list, but it's empty
    this.controller.setupWidget("feedListWgt",
         {
            itemTemplate:"feedList/feedRowTemplate", 
            listTemplate:"feedList/feedListTemplate", 
            addItemLabel:"Add...", 
            swipeToDelete:true, 
            renderLimit: 40,
            reorderable:true
        },
        this.feedWgtModel = {items: this.feeds.list});
    
    // Setup event handlers: list selection, add, delete and reorder feed entry
    this.showFeedHandler = this.showFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listTap,
        this.showFeedHandler);
    this.addNewFeedHandler = this.addNewFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listAdd,
        this.addNewFeedHandler);    
    this.listDeleteFeedHandler = this.listDeleteFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listDelete,
        this.listDeleteFeedHandler);
    this.listReorderFeedHandler = this.listReorderFeed.bindAsEventListener(this);
    this.controller.listen("feedListWgt", Mojo.Event.listReorder,
        this.listReorderFeedHandler); 
    
    // Update the feed list
    this.feeds.updateFeedList();
};

// activate
FeedListAssistant.prototype.activate =  function() {
    this.feeds.registerListModel(this, this.feedWgtModel);
    
    if (News.feedListChanged === true)    {
        this.feedWgtModel.items = this.feeds.list; 
        this.controller.modelChanged(this.feedWgtModel, this);
        this.controller.modelChanged(this.searchFieldModel, this);

    } 
};

// deactivate
FeedListAssistant.prototype.deactivate =  function() {
    Mojo.Log.info("FeedList deactivating"); 
    this.feeds.removeListModel(this.feedWgtModel);
};

// cleanup - always turn off timers, and save this.feeds.list contents
FeedListAssistant.prototype.cleanup =  function() {
    Mojo.Log.info("FeedList cleaning up");
    
    // Remove event listeners
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listTap, this.showFeedHandler);
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listAdd, this.addNewFeedHandler);    
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listDelete, this.listDeleteFeedHandler);
    this.controller.stopListening("feedListWgt", 
        Mojo.Event.listReorder, this.listReorderFeedHandler);
};

// ------------------------------------------------------------------------
// List functions for Delete, Reorder and Add
//
// listDeleteFeed - triggered by deleting a feed from the list and updates
// the feedlist to reflect the deletion
//
FeedListAssistant.prototype.listDeleteFeed =  function(event) {
    Mojo.log("News deleting "+event.item.title+".");
    
    var deleteIndex = this.feeds.list.indexOf(event.item);
    this.feeds.list.splice(deleteIndex, 1);
    this.feedWgtModel.items = this.feeds.list;
    News.feedListChanged = true;
};

// listReorderFeed - triggered re-ordering feed list and updates the
// feedlist to reflect the changed order
FeedListAssistant.prototype.listReorderFeed =  function(event) {
    Mojo.log("com.palm.app.news - News moving "+event.item.title+".");
    
    var fromIndex = this.feeds.list.indexOf(event.item);
    var toIndex = event.toIndex;
    this.feeds.list.splice(fromIndex, 1);
    this.feeds.list.splice(toIndex, 0, event.item);    
    this.feedWgtModel.items = this.feeds.list;
    News.feedListChanged = true;
};

// addNewFeed - triggered by "Add..." item in feed list
FeedListAssistant.prototype.addNewFeed = function() {
                
        this.controller.showDialog({
            template: "feedList/addFeed-dialog",
            assistant: new AddDialogAssistant(this, this.feeds)
        });
        
};

// ---------------------------------------------------------------------------------
// Show feed and popup menu handler
//
// showFeed - triggered by tapping a feed in the this.feeds.list.
//   Detects taps on the unReadCount icon; anywhere else, 
//   the scene for the list view is pushed. If the icon is tapped,
//   put up a submenu for the feedlist options
FeedListAssistant.prototype.showFeed = function(event) {
        var target = event.originalEvent.target.id;
        if (target !== "info") {
          Mojo.Controller.stageController.pushScene("storyList", 
            this.feeds.list, event.index);
        }
        else  {
            var myEvent = event;
            var findPlace = myEvent.originalEvent.target;
            this.popupIndex = event.index;
            this.controller.popupSubmenu({
              onChoose:  this.popupHandler,
              placeNear: findPlace,
              items: [
                {label: "All Unread", command: "feed-unread"},
                {label: "All Read", command: "feed-read"},
                {label: "Edit Feed", command: "feed-edit"}
                ]
              });
        }
};

// popupHandler - choose function for feedPopup
FeedListAssistant.prototype.popupHandler = function(command) {
        var popupFeed=this.feeds.list[this.popupIndex];    
        switch(command) {
            case "feed-unread":
                Mojo.Log.info("Popup - unread for feed:", 
                  popupFeed.title);
                
                for (var i=0; i<popupFeed.stories.length; i++ ) {
                    popupFeed.stories[i].unreadStyle = News.unreadStory;    
                }
                popupFeed.numUnRead = popupFeed.stories.length;
                this.controller.modelChanged(this.feedWgtModel);
                break;

            case "feed-read":
                Mojo.Log.info("Popup - read for feed:", 
                  popupFeed.title);
                for (var j=0; j<popupFeed.stories.length; j++ ) {
                    popupFeed.stories[j].unreadStyle = "";    
                }
                popupFeed.numUnRead = 0;
                this.controller.modelChanged(this.feedWgtModel);
                break;
                
            case "feed-edit":
                Mojo.Log.info("Popup edit for feed:", 
                  popupFeed.title);
                this.controller.showDialog({
                    template: "feedList/addFeed-dialog",
                    assistant: new AddDialogAssistant(this, 
                        this.feeds, this.popupIndex)
                });
                break;

       }
};