/*  StageAssistant - NEWS
    
    Copyright 2009 Palm, Inc.  All rights reserved.
    
    Responsible for app startup.
    Major components:
    - setup; app startup including preferences, initial load of feed data
        from the Depot and setting alarms for periodic feed updates
    - handleCommand; handles app menu selections
    
    Data structures:
    - globals; set of persistant data used throughout app
    - Feeds Model; handles all feedlist updates, db handling and default data
    - Cookies Model; handles saving and restoring preferences
    
    App architecture:
    - StageAssistant; handles startup, feed list management and app menu management
    - FeedListAssistant; handles feedList navigation, feedList search, feature feed
    - StoryListAssistant; handles single feed navigation
    - StoryViewAssistant; handles single story navigation
    - PreferencesAssistant; handles preferences display and changes
    - HelpAssistant; handles help topics display

*/  

//  ---------------------------------------------------------------
//    GLOBALS
//  ---------------------------------------------------------------

//  News namespace
News = {};

// Constants
News.unreadStory = "unReadStyle";                                    
News.versionString = "1.0";
News.MainStageName = "main";
News.errorNone =    "0";                    // No error, success
News.invalidFeedError = "1";                // Not RSS2, RDF (RSS1), or ATOM

// Global Data Structures

// Persistent Globals - will be saved across app launches
News.featureFeedEnable = false;             // Enables feed rotation
News.featureStoryInterval = 5000;           // Feature Interval (in ms) 
News.notificationEnable = true;             // Enables notifcations 
News.feedUpdateBackgroundEnable = false;    // Enable device wakeup 
News.feedUpdateInterval = 900000;           // Feed update interval 

//  Session Globals - not saved across app launches
News.feedListChanged = false;               // Triggers update to Depot db
News.feedListUpdateInProgress = false;      // Feed update is in progress
News.featureStoryTimer = null;              // Timer for story rotations
News.dbUpdate = "";                         // Default is no update
News.wakeupTaskId = undefined;              // Id for wakeup tasks

// Setup App Menu for all scenes; all menu actions handled in
//  StageAssistant.handleCommand()
News.MenuAttr = {omitDefaultItems: true};

News.MenuModel = {
    visible: true,
    items: [ 
        {label: "About News...", command: "do-aboutNews"},
        Mojo.Menu.editItem,
        {label: "Update All Feeds", checkEnabled: true, command: "do-feedUpdate"},
        {label: "Preferences...", command: "do-newsPrefs"},    
        Mojo.Menu.helpItem          
    ]
};

function StageAssistant () {

}

//  -------------------------------------------------------
//  setup - all startup actions:
//    - Setup globals with preferences
//    - Set up application menu; used in every scene
//    - Open Depot and use contents for feedList
//    - Initiate alarm for first feed update                    

StageAssistant.prototype.setup = function() {
    
    // initialize the feeds model
    this.feeds = new Feeds();
    this.feeds.loadFeedDb();
        
    // load preferences and globals from saved cookie
    News.Cookie.initialize();
    
    // Set up first timeout alarm
    this.setWakeup();

    this.controller.pushScene("feedList", this.feeds);
    
};

// -----------------------------------------
// handleCommand - called to handle app menu selections
//    
StageAssistant.prototype.handleCommand = function(event) {    
    if (event.type == Mojo.Event.commandEnable) {
        if (News.feedListUpdateInProgress && (event.command == "do-feedUpdate")) {
            event.preventDefault();
        }
    }
    
    else {
        
        if(event.type == Mojo.Event.command) {
            switch(event.command) {
            
                case "do-aboutNews":
                    this.controller.showAlertDialog({
                            onChoose: function(value) {},
                            title: "News — v#{version}".interpolate({version: News.versionString}),
                            message: "Copyright 2009, Palm Inc.",
                            choices:[
                                {label:"OK", value:""}   
                                       ]
                              });
                break;
                
                case "do-newsPrefs":
                    this.controller.pushScene("preferences");
                break;
            
                case "do-feedUpdate":
                    this.feeds.updateFeedList();
                break;
            }
        }
    }
};

// ------------------------------------------------------------------------
// handleWakeup - called when wakeup timer fires; sets a new timer and calls
//   for a feed update cycle
StageAssistant.prototype.handleWakeup = function() {    

    // Set next wakeup alarm
    this.setWakeup();

    // Update the feed list
    Mojo.Log.info("Update FeedList");
    this.feeds.updateFeedList();
    
};

// ------------------------------------------------------------------------
// setWakeup - called to setup the wakeup alarm for background feed updates
//   if preferences are not set for a manual update (value of 0)
StageAssistant.prototype.setWakeup = function() {    

	if (News.feedUpdateInterval !== 0)	{
	    var interval = News.feedUpdateInterval;
	    News.wakeupTaskId = this.controller.window.setTimeout(this.handleWakeup.bind(this), interval);
		Mojo.Log.info("Set Interval Timer: ", interval);
	}
    
};
